@@ -6,7 +6,7 @@
  * Descr.: operating system interface management - C part                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2021 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -1004,7 +1004,7 @@ Pl_Popen_3(WamWord cmd_word, WamWord mode_word, WamWord stm_word)
 {
   char *cmd;
   int atom;
-  int mode;
+  int mode = 0;			/* init for the compiler */
   int stm;
   FILE *f;
   char open_str[10];
@@ -1032,7 +1032,7 @@ Pl_Popen_3(WamWord cmd_word, WamWord mode_word, WamWord stm_word)
 
   sprintf(pl_glob_buff, "popen_stream('%.1024s')", cmd);
   atom = Pl_Create_Allocate_Atom(pl_glob_buff);
-  stm = Pl_Add_Stream_For_Stdio_Desc(f, atom, mode, TRUE);
+  stm = Pl_Add_Stream_For_Stdio_Desc(f, atom, mode, TRUE, FALSE);
   pl_stm_tbl[stm]->fct_close = (StmFct) pclose;
 
   return Pl_Get_Integer(stm, stm_word);
@@ -1082,13 +1082,13 @@ Pl_Exec_5(WamWord cmd_word, WamWord stm_in_word, WamWord stm_out_word,
   sprintf(pl_glob_buff, "exec_stream('%.1024s')", cmd);
   atom = Pl_Create_Allocate_Atom(pl_glob_buff);
 
-  stm = Pl_Add_Stream_For_Stdio_Desc(f_in, atom, STREAM_MODE_WRITE, TRUE);
+  stm = Pl_Add_Stream_For_Stdio_Desc(f_in, atom, STREAM_MODE_WRITE, TRUE, FALSE);
   Pl_Get_Integer(stm, stm_in_word);
 #ifdef DEBUG
   DBGPRINTF("Added Stream Input: %d\n", stm);
 #endif
 
-  stm = Pl_Add_Stream_For_Stdio_Desc(f_out, atom, STREAM_MODE_READ, TRUE);
+  stm = Pl_Add_Stream_For_Stdio_Desc(f_out, atom, STREAM_MODE_READ, TRUE, FALSE);
   pl_stm_tbl[stm]->prop.eof_action = STREAM_EOF_ACTION_RESET;
   Pl_Get_Integer(stm, stm_out_word);
 
@@ -1096,7 +1096,7 @@ Pl_Exec_5(WamWord cmd_word, WamWord stm_in_word, WamWord stm_out_word,
   DBGPRINTF("Added Stream Output: %d\n", stm);
 #endif
 
-  stm = Pl_Add_Stream_For_Stdio_Desc(f_err, atom, STREAM_MODE_READ, TRUE);
+  stm = Pl_Add_Stream_For_Stdio_Desc(f_err, atom, STREAM_MODE_READ, TRUE, FALSE);
   pl_stm_tbl[stm]->prop.eof_action = STREAM_EOF_ACTION_RESET;
   Pl_Get_Integer(stm, stm_err_word);
 #ifdef DEBUG
@@ -1130,14 +1130,14 @@ Pl_Create_Pipe_2(WamWord stm_in_word, WamWord stm_out_word)
   Os_Test_Error_Null((f_in = fdopen(p[0], "rt")));
   sprintf(pl_glob_buff, "pipe_stream_in");
   atom = Pl_Create_Allocate_Atom(pl_glob_buff);
-  stm = Pl_Add_Stream_For_Stdio_Desc(f_in, atom, STREAM_MODE_READ, TRUE);
+  stm = Pl_Add_Stream_For_Stdio_Desc(f_in, atom, STREAM_MODE_READ, TRUE, FALSE);
   pl_stm_tbl[stm]->prop.eof_action = STREAM_EOF_ACTION_RESET;
   Pl_Get_Integer(stm, stm_in_word);
 
   Os_Test_Error_Null((f_out = fdopen(p[1], "wt")));
   sprintf(pl_glob_buff, "pipe_stream_out");
   atom = Pl_Create_Allocate_Atom(pl_glob_buff);
-  stm = Pl_Add_Stream_For_Stdio_Desc(f_out, atom, STREAM_MODE_WRITE, TRUE);
+  stm = Pl_Add_Stream_For_Stdio_Desc(f_out, atom, STREAM_MODE_WRITE, TRUE, FALSE);
   Pl_Get_Integer(stm, stm_out_word);
 
   return TRUE;
@@ -1259,7 +1259,7 @@ Select_Init_Set(WamWord list_word, fd_set *set, int check)
       lst_adr = UnTag_LST(word);
       DEREF(Car(lst_adr), word, tag_mask);
       if (tag_mask == TAG_INT_MASK)
-	fd = Pl_Rd_Positive_Check(word);
+	fd = Pl_Rd_C_Int_Positive_Check(word);
       else
 	{
 	  stm = Pl_Get_Stream_Or_Alias(word, check);
@@ -1366,7 +1366,7 @@ Pl_Send_Signal_2(WamWord pid_word, WamWord signal_word)
   int atom;
   int i;
 
-  pid = Pl_Rd_Integer_Check(pid_word);
+  pid = Pl_Rd_C_Int_Check(pid_word);
 
   DEREF(signal_word, word, tag_mask);
   if (tag_mask == TAG_ATM_MASK)
@@ -1381,7 +1381,7 @@ Pl_Send_Signal_2(WamWord pid_word, WamWord signal_word)
 	  }
     }
   else
-    sig = Pl_Rd_Integer_Check(word);
+    sig = Pl_Rd_C_Int_Check(word);
 
 #ifdef _WIN32
   {
@@ -1420,7 +1420,7 @@ Pl_Wait_2(WamWord pid_word, WamWord status_word)
   int pid;
   int status;
 
-  pid = Pl_Rd_Integer_Check(pid_word);
+  pid = Pl_Rd_C_Int_Check(pid_word);
   Pl_Check_For_Un_Integer(status_word);
 
   status  = Pl_M_Get_Status(pid);
