@@ -6,7 +6,7 @@
  * Descr.: term support - header file                                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2021 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -50,22 +50,16 @@
 
 #ifdef TERM_SUPP_FILE
 
-WamWord pl_pi_module_word;
 WamWord pl_pi_name_word;
 WamWord pl_pi_arity_word;
 
-int pl_calling_module;
-
 PlLong pl_glob_dico_var[MAX_VAR_IN_TERM];	/* a general purpose dico */
 
 #else
 
-extern WamWord pl_pi_module_word;
 extern WamWord pl_pi_name_word;
 extern WamWord pl_pi_arity_word;
 
-extern int pl_calling_module;
-
 extern PlLong pl_glob_dico_var[];
 
 #endif
@@ -77,12 +71,14 @@ extern PlLong pl_glob_dico_var[];
  * Function Prototypes             *
  *---------------------------------*/
 
-#define Pl_Unset_Calling_Module()    (pl_calling_module = -1)
-
-#define Pl_Set_Calling_Module(m)     (pl_calling_module = (m))
-
 PlLong Pl_Term_Compare(WamWord start_u_word, WamWord start_v_word);
 
+Bool Is_List(WamWord start_word);
+
+Bool Is_Partial_List(WamWord start_word);
+
+Bool Is_List_Or_Partial(WamWord start_word);
+
 Bool Pl_Treat_Vars_Of_Term(WamWord start_word, Bool generic_var,
 			   Bool (*fct) ());
 
@@ -94,23 +90,7 @@ void Pl_Copy_Term(WamWord *dst_adr, WamWord *src_adr);
 
 void Pl_Copy_Contiguous_Term(WamWord *dst_adr, WamWord *src_adr);
 
+int Pl_Get_Pred_Indicator(WamWord pred_indic_word, Bool must_be_ground,
+			  int *arity);
+
 Bool Pl_Acyclic_Term_1(WamWord start_word);
-
-
-WamWord Pl_Strip_Module(WamWord term_word, Bool accept_var, Bool raise_error, 
-			WamWord *goal_word);
-
-int Pl_Strip_Module_Top(WamWord start_word, Bool accept_var, Bool raise_error,
-			WamWord *goal_word);
-
-WamWord Pl_Get_Pred_Indicator(WamWord pred_indic_word, Bool accept_var,
-			      int *func, int *arity);
-
-int Pl_Get_Pred_Indicator_Top(WamWord pred_indic_word, Bool accept_var, 
-			      int *func, int *arity);
-
-WamWord Pl_Get_Head_And_Body(WamWord clause_word, WamWord *head_word, WamWord *body_word);
-
-int Pl_Get_Head_And_Body_Top(WamWord clause_word, WamWord *head_word, WamWord *body_word);
-
-WamWord Pl_Term_To_Goal(WamWord term_word, WamWord module_word, WamWord call_info_word);
