@@ -6,7 +6,7 @@
  * Descr.: atom table management                                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2021 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -582,7 +582,7 @@ Pl_Gen_New_Atom(char *prefix)
 int
 Pl_Find_Next_Atom(int last_atom)
 {
-  while (++last_atom < pl_max_atom)
+  while ((PlULong) ++last_atom < pl_max_atom)
     {
       if (pl_atom_tbl[last_atom].name)
 	return last_atom;
