@@ -6,7 +6,7 @@
  * Descr.: machine dependent features                                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2021 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -190,7 +190,7 @@ Pl_Init_Machine(void)
 char *
 Pl_M_Sys_Err_String(int ret_val)
 {
-#ifdef M_sparc_sunos
+#ifdef M_sparc32_sunos
   extern char *sys_errlist[];
   extern int sys_nerr;
 #endif
@@ -220,7 +220,7 @@ Pl_M_Sys_Err_String(int ret_val)
 #endif
 
 
-#if defined(M_sparc_sunos)
+#if defined(M_sparc32_sunos)
   str = (errno >= 0 && errno < sys_nerr) ? sys_errlist[errno] : NULL;
 #else
   str = strerror(errno);
@@ -503,9 +503,8 @@ Pl_M_Host_Name_From_Adr(char *host_address)
   struct hostent *host_entry;
   struct in_addr iadr;
 
-#if defined(M_sparc_sunos) || defined(M_sparc_solaris) || \
-    defined(M_ix86_cygwin) || defined(M_ix86_solaris)  || \
-    defined(_WIN32)
+#if defined(M_sparc32_sunos) || defined(M_sparc32_solaris) || defined(M_ix86_solaris) || \
+    defined(_WIN32) || defined(__CYGWIN__)
   if ((iadr.s_addr = inet_addr(host_address)) == -1)
 #else
   if (inet_aton(host_address, &iadr) == 0)
@@ -591,16 +590,25 @@ Pl_M_Get_Working_Dir(void)
  *                                                                         *
  * returns an absolute file name.                                          *
  *-------------------------------------------------------------------------*/
+#ifdef __GNUC__
+#pragma GCC diagnostic push
+#ifdef __clang__
+#pragma GCC diagnostic ignored "-Wunknown-warning-option"
+#endif
+#pragma GCC diagnostic ignored "-Wformat-overflow"
+#endif
 char *
 Pl_M_Absolute_Path_Name(char *src)
 {
-  static char buff[2][MAXPATHLEN];
-  int res = 0;
-  char *dst;
+  static char buff1[MAXPATHLEN];
+  static char buff2[MAXPATHLEN];
+  char *dst, *base_dst;
   char *p, *q;
   char c;
 
-  dst = buff[res];
+#define SWAP_SRC_DST { char *tmp = src; src = dst; dst = tmp; }
+
+  dst = buff1;
   while ((*dst++ = *src))       /* expand $VARNAME and %VARNAME% (Win32) */
     {
       c = *src++;
@@ -608,7 +616,7 @@ Pl_M_Absolute_Path_Name(char *src)
 #if defined(_WIN32) || defined(__CYGWIN__)
           || c == '%'
 #endif
-        )
+	  )
         {
           p = dst;
           while (isalnum(*src) || *src == '_')
@@ -637,9 +645,11 @@ Pl_M_Absolute_Path_Name(char *src)
     }
   *dst = '\0';
 
-  if (buff[res][0] == '~')
+  src = buff1;
+  dst = buff2;
+  if (src[0] == '~')
     {
-      if (Is_Dir_Sep(buff[res][1]) || buff[res][1] == '\0') /* ~/... cf $HOME */
+      if (Is_Dir_Sep(src[1]) || src[1] == '\0') /* ~/... cf $HOME */
         {
 	  q = NULL;;
           if ((p = getenv("HOME")) == NULL)
@@ -654,41 +664,42 @@ Pl_M_Absolute_Path_Name(char *src)
 	    }
 	  if (q == NULL)
 	    q = "";
-          sprintf(buff[1 - res], "%s%s/%s", q, p, buff[res] + 1);
-          res = 1 - res;
+          sprintf(dst, "%s%s/%s", q, p, src + 1);
+	  SWAP_SRC_DST;
         }
 #if defined(__unix__) || defined(__CYGWIN__)
       else                      /* ~user/... read passwd */
         {
           struct passwd *pw;
 
-          p = buff[res] + 1;
+          p = src + 1;
           while (*p && !Is_Dir_Sep(*p))
             p++;
 
-          buff[res][0] = *p;
+          src[0] = *p;
           *p = '\0';
-          if ((pw = getpwnam(buff[res] + 1)) == NULL)
+          if ((pw = getpwnam(src + 1)) == NULL)
             return NULL;
 
-          *p = buff[res][0];
+          *p = src[0];
 
-          sprintf(buff[1 - res], "%s/%s", pw->pw_dir, p);
-          res = 1 - res;
+          sprintf(dst, "%s/%s", pw->pw_dir, p);
+	  SWAP_SRC_DST;
         }
 #endif
     }
 
-  if (strcmp(buff[res], "user") == 0)   /* prolog special file 'user' */
-    return buff[res];
+  if (strcmp(src, "user") == 0)   /* prolog special file 'user' */
+    return src;
 
 #if defined(_WIN32) && !defined(__CYGWIN__)
 
-  if (_fullpath(buff[1 - res], buff[res], MAXPATHLEN) == NULL)
+  base_dst = dst;
+  if (_fullpath(dst, src, MAXPATHLEN) == NULL)
     return NULL;
 
-  res = 1 - res;
-  for (dst = buff[res]; *dst; dst++)    /* \ becomes / */
+  SWAP_SRC_DST;
+  for (dst = src; *dst; dst++)    /* \ becomes / */
     if (*dst == '\\')
       *dst = '/';
 
@@ -696,27 +707,20 @@ Pl_M_Absolute_Path_Name(char *src)
 
 #else  /* __unix__ || __CYGWIN__ */
 
-#if defined(__CYGWIN__)
+#if defined(__CYGWIN__) && !defined(__MSYS__)
 
-#if 0
-  cygwin_conv_to_full_posix_path(buff[res], buff[1 - res]);
-#else
-  cygwin_conv_path(CCP_WIN_A_TO_POSIX, buff[1 - res], buff[res], MAXPATHLEN);
-#endif
-  res = 1 - res;
+  cygwin_conv_pathx(CCP_WIN_A_TO_POSIX, dst, src, MAXPATHLEN);
+  SWAP_SRC_DST;
 
 #endif
 
-  if (buff[res][0] != '/')      /* add current directory */
+  if (src[0] != '/')      /* add current directory */
     {
-      sprintf(buff[1 - res], "%s/%s", Pl_M_Get_Working_Dir(), buff[res]);
-      res = 1 - res;
+      sprintf(dst, "%s/%.*s", Pl_M_Get_Working_Dir(), MAXPATHLEN, src); /* precise MAXPATHLEN to avoid gcc warning */
+      SWAP_SRC_DST;
     }
 
-  src = buff[res];
-  res = 1 - res;
-  dst = buff[res];
-
+  base_dst = dst;
   while ((*dst++ = *src))
     {
       if (*src++ != '/')
@@ -739,14 +743,12 @@ Pl_M_Absolute_Path_Name(char *src)
         continue;
       /* case /../ */
       src += 2;
-      p = dst - 2;
-      while (p >= buff[res] && *p != '/')
-        p--;
+      dst -= 2;
+      while (dst >= base_dst && *dst != '/')
+        dst--;
 
-      if (p < buff[res])
+      if (dst < base_dst)
         return NULL;
-
-      dst = p;
     }
 
   dst--;                        /* dst points the \0 */
@@ -760,11 +762,17 @@ Pl_M_Absolute_Path_Name(char *src)
 #define MIN_PREFIX 1            /* unix  minimal path /    */
 #endif
 
-  if (dst - buff[res] > MIN_PREFIX && Is_Dir_Sep(dst[-1]))
+  if (dst - base_dst < MIN_PREFIX) /* all removed, e.g. with /src/../ then add / */
+    strcpy(dst, "/");
+
+  if (dst - base_dst > MIN_PREFIX && Is_Dir_Sep(dst[-1]))
     dst[-1] = '\0';             /* remove last / or \ */
 
-  return buff[res];
+  return base_dst;
 }
+#ifdef __GNUC__
+#pragma GCC diagnostic pop
+#endif
 
 
 
