@@ -6,7 +6,7 @@
  * Descr.: stack hardware overflow detection (SIGSEGV)                     *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2021 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -56,9 +56,9 @@
 
 #include "gp_config.h"          /* ensure __unix__ defined if not Win32 */
 
-#if defined(__unix__) || defined(__CYGWIN__)
+#if !defined(__MSYS__) && (defined(__unix__) || defined(__CYGWIN__))
 #include <unistd.h>
-#elif defined(_WIN32)
+#elif defined(_WIN32) || defined(__MSYS__)
 #include <windows.h>
 #endif
 
@@ -154,7 +154,7 @@ static char *Stack_Overflow_Err_Msg(int stk_nb);
 
 
 
-#if defined(_WIN32)
+#if defined(_WIN32) || defined(__MSYS__)
 
 /*-------------------------------------------------------------------------*
  * GETPAGESIZE                                                             *
@@ -179,7 +179,7 @@ getpagesize(void)
 static void *
 Virtual_Mem_Alloc(void *addr, int length)
 {
-#if defined(_WIN32)
+#if defined(_WIN32) || defined(__MSYS__)
 
   addr = (void *) VirtualAlloc(addr, length,
 			       MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE);
@@ -230,7 +230,7 @@ Virtual_Mem_Alloc(void *addr, int length)
 static void
 Virtual_Mem_Free(void *addr, int length)
 {
-#if defined(_WIN32)
+#if defined(_WIN32) || defined(__MSYS__)
 
   if (!VirtualFree(addr, 0, MEM_RELEASE))
     Pl_Fatal_Error(ERR_CANNOT_FREE, GetLastError());
@@ -258,7 +258,7 @@ static void
 Virtual_Mem_Protect(void *addr, int length)
 {
   WamWord *end = (WamWord *) addr;
-#if defined(_WIN32)
+#if defined(_WIN32) || defined(__MSYS__)
   DWORD old_prot;
 
   if (!VirtualProtect(addr, length, PAGE_NOACCESS, &old_prot))
@@ -272,6 +272,10 @@ Virtual_Mem_Protect(void *addr, int length)
     if (munmap((void *) addr, length) == -1)
       Pl_Fatal_Error(ERR_CANNOT_UNMAP, Pl_M_Sys_Err_String(-1));
 
+#endif
+
+#ifdef DEBUG
+  DBGPRINTF("Protect at %p len: %d\n", addr, length);
 #endif
   end[-16] = M_MAGIC1;
   end[-32] = M_MAGIC2;           /* and rest (end[-1,...]) should be 0 */
@@ -370,19 +374,25 @@ Pl_Allocate_Stacks(void)
 
   Install_SIGSEGV_Handler();	/* install the real (and unique) SIGSEGV handler */
   Pl_Push_SIGSEGV_Handler(Default_SIGSEGV_Handler); /* install initial user SIGSEGV handler */
+
+#if 0 /* cause an exception */
+  addr -= page_size;
+  addr = pl_stk_tbl[1].stack - 128;
+  *addr = 123;
+#endif
 }
 
 
 
 
-#if defined(__unix__) || defined(__CYGWIN__)|| defined(_WIN64)
+#if !defined(__MSYS__) && (defined(__unix__) || defined(__CYGWIN__))
 
 /*-------------------------------------------------------------------------*
  * SIGSEGV_HANDLER                                                         *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 #if defined(HAVE_WORKING_SIGACTION) || \
-  defined(M_sparc_solaris) || defined(M_ix86_solaris) || defined(M_x86_64_solaris)
+  defined(M_sparc32_solaris) || defined(M_ix86_solaris) || defined(M_x86_64_solaris)
 
 static void
 SIGSEGV_Handler(int sig, siginfo_t *sip)
@@ -391,7 +401,7 @@ SIGSEGV_Handler(int sig, siginfo_t *sip)
   Handle_Bad_Address(addr);
 }
 
-#elif defined(M_sparc_sunos)
+#elif defined(M_sparc32_sunos)
 
 static void
 SIGSEGV_Handler(int sig, int code, int scp, void *addr)
@@ -427,7 +437,7 @@ SIGSEGV_Handler(int sig, struct sigcontext scp)
   Handle_Bad_Address(addr);
 }
 
-#elif defined(M_powerpc_linux)
+#elif defined(M_ppc32_linux)
 
 static void
 SIGSEGV_Handler(int sig, struct sigcontext scp)
@@ -446,7 +456,7 @@ SIGSEGV_Handler(int sig, siginfo_t * si)
   Handle_Bad_Address(addr);
 }
 
-#elif defined(M_ix86_bsd) || defined(M_powerpc_bsd) || defined(M_sparc_bsd) || defined(M_sparc64_bsd)
+#elif defined(M_ix86_bsd) || defined(M_ppc32_bsd) || defined(M_sparc32_bsd) || defined(M_sparc64_bsd)
 static void
 SIGSEGV_Handler(int sig, int code, struct sigcontext *scp)
 {
@@ -463,7 +473,7 @@ SIGSEGV_Handler(int sig, siginfo_t *sip, void *scp)
   Handle_Bad_Address(addr);
 }
 
-#elif defined(M_mips_irix)
+#elif defined(M_mips32_irix)
 
 static void
 SIGSEGV_Handler(int sig, int code, struct sigcontext *scp)
@@ -495,7 +505,7 @@ SIGSEGV_Handler(int sig)	/* cannot detect fault addr */
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static LONG WINAPI 
-Win32_Exception_Handler(LPEXCEPTION_POINTERS ei)
+Win32_Exception_Handler(PEXCEPTION_POINTERS ei)
 {
   void *addr;
   switch(ei->ExceptionRecord->ExceptionCode)  
@@ -528,9 +538,9 @@ Win32_Exception_Handler(LPEXCEPTION_POINTERS ei)
 static void
 Install_SIGSEGV_Handler(void)
 {
-#if defined(HAVE_WORKING_SIGACTION) || \
-  defined(M_sparc_solaris) || defined(M_ix86_solaris) || \
-  defined(M_ix86_sco) || defined(M_x86_64_solaris)
+#if !defined(__MSYS__) && (defined(HAVE_WORKING_SIGACTION) || \
+  defined(M_sparc32_solaris) || defined(M_ix86_solaris) || \
+  defined(M_ix86_sco) || defined(M_x86_64_solaris))
 
   struct sigaction act;
 
@@ -543,9 +553,10 @@ Install_SIGSEGV_Handler(void)
   sigaction(SIGBUS, &act, NULL);
 #  endif
 
-#elif defined(_WIN32) && !defined(_WIN64)
+#elif defined(_WIN32) || defined(__MSYS__)
 
-  SetUnhandledExceptionFilter(Win32_Exception_Handler);
+  AddVectoredExceptionHandler(1, Win32_Exception_Handler);
+  //  SetUnhandledExceptionFilter(Win32_Exception_Handler);
 
 #else
 
